==> umi-plugin-dva-0.6.3.tgz: package/src/index.js <==
import { readFileSync, writeFileSync, existsSync, statSync } from 'fs';
import { join, dirname } from 'path';
import globby from 'globby';
import uniq from 'lodash.uniq';

export default function(api, opts = {}) {
  const { RENDER, ROUTER_MODIFIER, IMPORT } = api.placeholder;
  const { paths, config } = api.service;
  const { winPath } = api.utils;
  const dvaContainerPath = join(paths.absTmpDirPath, 'DvaContainer.js');
  const isProduction = process.env.NODE_ENV === 'production';
  const shouldImportDynamic = isProduction && !config.disableDynamicImport;

  function getDvaJS() {
    if (existsSync(join(paths.absSrcPath, 'dva.js'))) {
      return winPath(join(paths.absSrcPath, 'dva.js'));
    }
    if (existsSync(join(paths.absSrcPath, 'dva.ts'))) {
      return winPath(join(paths.absSrcPath, 'dva.ts'));
    }
  }

  function getModel(cwd) {
    const modelJSPath = join(cwd, 'model.js');
    if (existsSync(modelJSPath)) {
      return [winPath(modelJSPath)];
    }
    const modelTSPath = join(cwd, 'model.ts');
    if (existsSync(modelTSPath)) {
      return [winPath(modelTSPath)];
    }

    return globby
      .sync(`./${config.singular ? 'model' : 'models'}/**/*.{ts,js}`, {
        cwd,
      })
      .filter(p => !p.endsWith('.d.ts'))
      .map(p => winPath(join(cwd, p)));
  }

  function endWithSlash(path) {
    return path.slice(-1) !== '/' ? `${path}/` : path;
  }

  function isPagesPath(path) {
    return (
      winPath(endWithSlash(path)) === winPath(endWithSlash(paths.absPagesPath))
    );
  }

  function isSrcPath(path) {
    return (
      winPath(endWithSlash(path)) === winPath(endWithSlash(paths.absSrcPath))
    );
  }

  function getModelsWithRoutes(routes) {
    return routes.reduce((memo, route) => {
      return [
        ...memo,
        ...getPageModels(join(paths.cwd, route.component)),
        ...(route.routes ? getModelsWithRoutes(route.routes) : []),
      ];
    }, []);
  }

  function getGlobalModels() {
    let models = getModel(paths.absSrcPath);
    if (!shouldImportDynamic) {
      // dev 模式下还需要额外载入 page 路由的 models 文件
      models = [...models, ...getModelsWithRoutes(api.service.routes)];
      // 去重
      models = uniq(models);
    }
    return models;
  }

  function getPageModels(cwd) {
    let models = [];
    while (!isPagesPath(cwd) && !isSrcPath(cwd) && cwd !== '/') {
      models = models.concat(getModel(cwd));
      cwd = dirname(cwd);
    }
    return models;
  }

  function getGlobalModelContent() {
    return getGlobalModels()
      .map(path =>
        `
    app.model({ ...(require('${path}').default) });
  `.trim(),
      )
      .join('\r\n');
  }

  function getPluginContent() {
    const pluginPaths = globby.sync('plugins/**/*.{js,ts}', {
      cwd: paths.absSrcPath,
    });
    const ret = pluginPaths.map(path =>
      `
app.use(require('../../${path}').default);
  `.trim(),
    );
    if (opts.immer) {
      ret.push(
        `
app.use(require('${winPath(require.resolve('dva-immer'))}').default());
      `.trim(),
      );
    }
    return ret.join('\r\n');
  }

  function stripFirstSlash(path) {
    if (path.charAt(0) === '/') {
      return path.slice(1);
    } else {
      return path;
    }
  }

  function chunkName(path) {
    return stripFirstSlash(
      winPath(path).replace(winPath(paths.cwd), ''),
    ).replace(/\//g, '__');
  }

  api.register('generateFiles', () => {
    const tpl = join(__dirname, '../template/DvaContainer.js');
    let tplContent = readFileSync(tpl, 'utf-8');
    const dvaJS = getDvaJS();
    if (dvaJS) {
      tplContent = tplContent.replace(
        '<%= ExtendDvaConfig %>',
        `
...((require('${dvaJS}').config || (() => ({})))()),
        `.trim(),
      );
      //         .replace('<%= EnhanceApp %>', `
      // app = (require('${dvaJS}').enhance || (app => app))(app);
      //         `.trim());
    }
    tplContent = tplContent
      .replace('<%= ExtendDvaConfig %>', '')
      .replace('<%= EnhanceApp %>', '')
      .replace('<%= RegisterPlugins %>', getPluginContent())
      .replace('<%= RegisterModels %>', getGlobalModelContent());
    writeFileSync(dvaContainerPath, tplContent, 'utf-8');
  });

  api.register('modifyRouterFile', ({ memo }) => {
    return memo
      .replace(
        IMPORT,
        `
import { routerRedux } from 'dva/router';
${shouldImportDynamic ? `import _dvaDynamic from 'dva/dynamic';` : ''}
${IMPORT}
      `.trim(),
      )
      .replace(
        ROUTER_MODIFIER,
        `
const { ConnectedRouter } = routerRedux;
Router = ConnectedRouter;
${ROUTER_MODIFIER}
      `.trim(),
      );
  });

  if (shouldImportDynamic) {
    api.register('modifyRouteComponent', ({ memo, args }) => {
      const { pageJSFile, webpackChunkName } = args;
      if (!webpackChunkName) {
        return memo;
      }

      let ret = `
_dvaDynamic({
  <%= MODELS %>
  component: () => import(/* webpackChunkName: '${webpackChunkName}' */'${pageJSFile}'),
})
      `.trim();
      const models = getPageModels(join(paths.absTmpDirPath, pageJSFile));
      if (models && models.length) {
        ret = ret.replace(
          '<%= MODELS %>',
          `
app: window.g_app,
models: () => [
  ${models
    .map(
      model =>
        `import(/* webpackChunkName: '${chunkName(model)}' */'${model}')`,
    )
    .join(',\r\n  ')}
],
      `.trim(),
        );
      }
      return ret.replace('<%= MODELS %>', '');
    });
  }

  api.register('modifyEntryFile', ({ memo }) => {
    return memo.replace(
      RENDER,
      `
const DvaContainer = require('./DvaContainer').default;
ReactDOM.render(React.createElement(
  DvaContainer,
  null,
  React.createElement(require('./router').default)
), document.getElementById('root'));
      `.trim(),
    );
  });

  api.register('modifyAFWebpackOpts', ({ memo }) => {
    memo.alias = {
      ...memo.alias,
      dva: dirname(require.resolve('dva/package')),
      'dva-loading': require.resolve('dva-loading'),
      'path-to-regexp': require.resolve('path-to-regexp'),
      'object-assign': require.resolve('object-assign'),
      ...(opts.immer
        ? {
            immer: require.resolve('immer'),
          }
        : {}),
    };
    return memo;
  });

  api.register('modifyPageWatchers', ({ memo }) => {
    return [
      ...memo,
      join(paths.absSrcPath, 'models'),
      join(paths.absSrcPath, 'plugins'),
      join(paths.absSrcPath, 'model.js'),
      join(paths.absSrcPath, 'model.ts'),
      join(paths.absSrcPath, 'dva.js'),
      join(paths.absSrcPath, 'dva.ts'),
    ];
  });
}

==> umi-plugin-dva-0.7.2.tgz: package/src/index.js <==
import { readFileSync, writeFileSync, existsSync } from 'fs';
import { join, dirname } from 'path';
import globby from 'globby';
import uniq from 'lodash.uniq';
import isRoot from 'path-is-root';

export function getModel(cwd, api) {
  const { config } = api.service;
  const { winPath } = api.utils;

  const modelJSPath = join(cwd, 'model.js');
  if (existsSync(modelJSPath)) {
    return [winPath(modelJSPath)];
  }
  const modelJSXPath = join(cwd, 'model.jsx');
  if (existsSync(modelJSXPath)) {
    return [winPath(modelJSXPath)];
  }
  const modelTSPath = join(cwd, 'model.ts');
  if (existsSync(modelTSPath)) {
    return [winPath(modelTSPath)];
  }

  const modelTSXPath = join(cwd, 'model.tsx');
  if (existsSync(modelTSXPath)) {
    return [winPath(modelTSXPath)];
  }

  return globby
    .sync(`./${config.singular ? 'model' : 'models'}/**/*.{ts,tsx,js,jsx}`, {
      cwd,
    })
    .filter(
      p =>
        !p.endsWith('.d.ts') &&
        !p.endsWith('.test.js') &&
        !p.endsWith('.test.jsx') &&
        !p.endsWith('.test.ts') &&
        !p.endsWith('.test.tsx'),
    )
    .map(p => winPath(join(cwd, p)));
}

export default function(api, opts = {}) {
  const { RENDER, ROUTER_MODIFIER, IMPORT } = api.placeholder;
  const { paths, config } = api.service;
  const { winPath } = api.utils;
  const dvaContainerPath = join(paths.absTmpDirPath, 'DvaContainer.js');
  const isProduction = process.env.NODE_ENV === 'production';
  const shouldImportDynamic = isProduction && !config.disableDynamicImport;

  function getDvaJS() {
    if (existsSync(join(paths.absSrcPath, 'dva.js'))) {
      return winPath(join(paths.absSrcPath, 'dva.js'));
    }
    if (existsSync(join(paths.absSrcPath, 'dva.ts'))) {
      return winPath(join(paths.absSrcPath, 'dva.ts'));
    }
  }

  function endWithSlash(path) {
    return path.slice(-1) !== '/' ? `${path}/` : path;
  }

  function isPagesPath(path) {
    return (
      endWithSlash(winPath(path)) === endWithSlash(winPath(paths.absPagesPath))
    );
  }

  function isSrcPath(path) {
    return (
      endWithSlash(winPath(path)) === endWithSlash(winPath(paths.absSrcPath))
    );
  }

  function getModelsWithRoutes(routes) {
    return routes.reduce((memo, route) => {
      return [
        ...memo,
        ...getPageModels(join(paths.cwd, route.component)),
        ...(route.routes ? getModelsWithRoutes(route.routes) : []),
      ];
    }, []);
  }

  function getGlobalModels() {
    let models = getModel(paths.absSrcPath, api);
    if (!shouldImportDynamic) {
      // dev 模式下还需要额外载入 page 路由的 models 文件
      models = [...models, ...getModelsWithRoutes(api.service.routes)];
      // 去重
      models = uniq(models);
    }
    return models;
  }

  function getPageModels(cwd) {
    let models = [];
    while (!isPagesPath(cwd) && !isSrcPath(cwd) && !isRoot(cwd)) {
      models = models.concat(getModel(cwd, api));
      cwd = dirname(cwd);
    }
    return models;
  }

  function getModelName(model) {
    const modelArr = winPath(model).split('/');
    return modelArr[modelArr.length - 1];
  }

  function exclude(models, excludes) {
    return models.filter(model => {
      for (const exclude of excludes) {
        if (typeof exclude === 'function' && exclude(getModelName(model))) {
          return false;
        }
        if (exclude instanceof RegExp && exclude.test(getModelName(model))) {
          return false;
        }
      }
      return true;
    });
  }

  function optsToArray(item) {
    if (!item) return [];
    if (Array.isArray(item)) {
      return item;
    } else {
      return [item];
    }
  }

  function getGlobalModelContent() {
    return exclude(getGlobalModels(), optsToArray(opts.exclude))
      .map(path =>
        `
    app.model({ ...(require('${path}').default) });
  `.trim(),
      )
      .join('\r\n');
  }

  function getPluginContent() {
    const pluginPaths = globby.sync('plugins/**/*.{js,ts}', {
      cwd: paths.absSrcPath,
    });
    const ret = pluginPaths.map(path =>
      `
app.use(require('../../${path}').default);
  `.trim(),
    );
    if (opts.immer) {
      ret.push(
        `
app.use(require('${winPath(require.resolve('dva-immer'))}').default());
      `.trim(),
      );
    }
    return ret.join('\r\n');
  }

  function stripFirstSlash(path) {
    if (path.charAt(0) === '/') {
      return path.slice(1);
    } else {
      return path;
    }
  }

  function chunkName(path) {
    return stripFirstSlash(
      winPath(path).replace(winPath(paths.cwd), ''),
    ).replace(/\//g, '__');
  }

  api.register('generateFiles', () => {
    const tpl = join(__dirname, '../template/DvaContainer.js');
    let tplContent = readFileSync(tpl, 'utf-8');
    const dvaJS = getDvaJS();
    if (dvaJS) {
      tplContent = tplContent.replace(
        '<%= ExtendDvaConfig %>',
        `
...((require('${dvaJS}').config || (() => ({})))()),
        `.trim(),
      );
      //         .replace('<%= EnhanceApp %>', `
      // app = (require('${dvaJS}').enhance || (app => app))(app);
      //         `.trim());
    }
    tplContent = tplContent
      .replace('<%= ExtendDvaConfig %>', '')
      .replace('<%= EnhanceApp %>', '')
      .replace('<%= RegisterPlugins %>', getPluginContent())
      .replace('<%= RegisterModels %>', getGlobalModelContent());
    writeFileSync(dvaContainerPath, tplContent, 'utf-8');
  });

  api.register('modifyRouterFile', ({ memo }) => {
    return memo
      .replace(
        IMPORT,
        `
import { routerRedux } from 'dva/router';
${shouldImportDynamic ? `import _dvaDynamic from 'dva/dynamic';` : ''}
${IMPORT}
      `.trim(),
      )
      .replace(
        ROUTER_MODIFIER,
        `
const { ConnectedRouter } = routerRedux;
Router = ConnectedRouter;
${ROUTER_MODIFIER}
      `.trim(),
      );
  });

  if (shouldImportDynamic) {
    api.register('modifyRouteComponent', ({ memo, args }) => {
      const { pageJSFile, webpackChunkName } = args;
      if (!webpackChunkName) {
        return memo;
      }

      let ret = `
_dvaDynamic({
  <%= MODELS %>
  component: () => import(/* webpackChunkName: '${webpackChunkName}' */'${pageJSFile}'),
})
      `.trim();
      const models = getPageModels(join(paths.absTmpDirPath, pageJSFile));
      if (models && models.length) {
        ret = ret.replace(
          '<%= MODELS %>',
          `
app: window.g_app,
models: () => [
  ${models
    .map(
      model =>
        `import(/* webpackChunkName: '${chunkName(model)}' */'${model}')`,
    )
    .join(',\r\n  ')}
],
      `.trim(),
        );
      }
      return ret.replace('<%= MODELS %>', '');
    });
  }

  api.register('modifyEntryFile', ({ memo }) => {
    const dvaRender = api.service.applyPlugins('modifyDvaRender', {
      initialValue: `
ReactDOM.render(React.createElement(
  DvaContainer,
  null,
  React.createElement(require('./router').default)
), document.getElementById('root'));
`.trim(),
    });

    return memo.replace(
      RENDER,
      `
const DvaContainer = require('./DvaContainer').default;
${dvaRender}
`.trim(),
    );
  });

  api.register('modifyAFWebpackOpts', ({ memo }) => {
    memo.alias = {
      ...memo.alias,
      dva: dirname(require.resolve('dva/package')),
      'dva-loading': require.resolve('dva-loading'),
      'path-to-regexp': require.resolve('path-to-regexp'),
      'object-assign': require.resolve('object-assign'),
      ...(opts.immer
        ? {
            immer: require.resolve('immer'),
          }
        : {}),
    };
    return memo;
  });

  api.register('modifyPageWatchers', ({ memo }) => {
    return [
      ...memo,
      join(paths.absSrcPath, 'models'),
      join(paths.absSrcPath, 'plugins'),
      join(paths.absSrcPath, 'model.js'),
      join(paths.absSrcPath, 'model.jsx'),
      join(paths.absSrcPath, 'model.ts'),
      join(paths.absSrcPath, 'model.tsx'),
      join(paths.absSrcPath, 'dva.js'),
      join(paths.absSrcPath, 'dva.ts'),
    ];
  });
}
